#!/usr/bin/python

from glob import glob
import os.path
import sys

CPS = 100 # System depended

def get_timing(filename):
	st = open(filename, "r").readline().split(" ");
	return (int(st[0]), int(st[1]), os.path.basename(filename))

first = True
timings = []
for i in glob("/tmp/boot_prof/*"):
	t = get_timing(i)
	timings.append(t)
	if first:
		first = False
		mint = t[0]
		maxt = t[1]
	else:
		mint = min(mint, t[0])
		maxt = max(maxt, t[1])

timings.sort(lambda a,b: a[0]-b[0])
timings = map(lambda i: (float(i[0]-mint)/CPS, float(i[1]-i[0])/CPS, i[2]), timings)
end_timings = float(maxt - mint)/CPS

from  PyQt4 import QtGui
#from  PyQt4 import QtOpenGL
from  PyQt4 import QtCore
#import mainw

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(QtCore.QSize(QtCore.QRect(0,0,800,470).size()).expandedTo(MainWindow.minimumSizeHint()))

        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")

        self.gridlayout = QtGui.QGridLayout(self.centralwidget)
        self.gridlayout.setObjectName("gridlayout")
        MainWindow.setCentralWidget(self.centralwidget)

        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0,0,800,30))
        self.menubar.setObjectName("menubar")

        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)

        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.actionSave = QtGui.QAction(MainWindow)
        self.actionSave.setObjectName("actionSave")
        self.menuFile.addAction(self.actionSave)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Boot timings analiser", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setText(QtGui.QApplication.translate("MainWindow", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+S", None, QtGui.QApplication.UnicodeUTF8))

#class PlotWidget(QtOpenGL.QGLWidget):
class PlotWidget(QtGui.QWidget):
	def __init__(self, *param):
#		QtOpenGL.QGLWidget.__init__(self, *param)
		QtGui.QWidget.__init__(self, *param)
		self.font = QtGui.QFont("Arial", 8)
		fm = QtGui.QFontMetrics(self.font)
		self.border = 20
		self.border_right = 180
		
		#height solve
		self.height = self.border
		self.height += (fm.height() + 1)*len(timings)
		self.axe_hpos = self.height
		self.height += fm.height() + 6
		self.height += self.border
		
		self.setMinimumHeight(self.height)
	def paintEvent(self, event):
		p = QtGui.QPainter(self)
		p.setFont(self.font)
		p.setPen(QtGui.QPen(QtGui.QBrush(QtCore.Qt.black), 1))
		fm = p.fontMetrics()
		
		p.fillRect(self.rect(), QtGui.QBrush(QtCore.Qt.white))
		
		h = self.border
		scale = float(self.rect().width() - self.border - self.border_right)/end_timings
		for i in timings:
			start, width = i[0]*scale + self.border, i[1]*scale
			end = start + width

			p.fillRect(start, h, width, fm.height(), QtGui.QBrush(QtGui.QColor(250,150,150)))
			p.drawText(start, h, 200, fm.height(), QtCore.Qt.AlignLeft, "%s (%.2f, %.2f)" % (i[2], i[0], i[1]))
			
			h += fm.height() + 1
		
		#p.drawLine(self.left_padding, self.str_h, self.width() - self.right_padding, self.str_h)
		self.drawAxe(p)
		p.end()
	def drawAxe(self, p):
		scale = float(self.rect().width() - self.border - self.border_right)/end_timings
		fm = p.fontMetrics()
		p.drawLine(self.border, self.axe_hpos + 2, self.border + end_timings*scale, self.axe_hpos + 2)
		time = 0.
		while time <= end_timings:
			coord = time*scale + self.border
			p.drawLine(coord, self.axe_hpos + 1, coord, self.axe_hpos + 4)
			p.drawText(coord - 50, self.axe_hpos + 5 + fm.ascent(), 100, fm.height(), QtCore.Qt.AlignCenter, "%.2f" % (time))
			time += end_timings/15 - 0.00001

class SaveDialog(QtGui.QFileDialog):
	def __init__(self, parent):
		QtGui.QFileDialog.__init__(self, parent)
		self.setAcceptMode(self.AcceptSave)
		self.setConfirmOverwrite(True)
		self.setDefaultSuffix("png")
		self.setFileMode(self.AnyFile)
		self.setReadOnly(True)
		self.setResolveSymlinks(True)
		self.setFilters(["Portable Network Graphics (*.png)", "Any file (*.*)"])

class MainWindow(QtGui.QMainWindow):
	def __init__(self):
		QtGui.QMainWindow.__init__(self)
		#mainw.Ui_MainWindow().setupUi(self)
		Ui_MainWindow().setupUi(self)
		self.pw = PlotWidget(self)
		self.setCentralWidget(self.pw)
		self.sd = SaveDialog(self)
	@QtCore.pyqtSignature("")
	def on_actionSave_activated(self):
		if self.sd.exec_():
			filename = self.sd.selectedFiles()[0]
			pm = QtGui.QPixmap.grabWidget(self.pw, self.pw.rect())
			pm.save(filename, "PNG")

app = QtGui.QApplication(sys.argv)
mainwindow = MainWindow()

mainwindow.show()
app.exec_()